@@ -1,10 +1,14 @@
 /* source: compat.h */
-/* Copyright Gerhard Rieger 2001-2009 */
+/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __compat_h_included
 #define __compat_h_included 1
 
+#if !HAVE_DECL_ENVIRON && HAVE_VAR_ENVIRON
+extern char **environ;
+#endif
+
 /*****************************************************************************/
 /* I dont like this system dependent part, but it would be quite a challenge
    for configure */
@@ -31,6 +35,10 @@
 
 /* substitute some features that might be missing on some platforms */
 
+#if !HAVE_TYPE_SIG_ATOMIC_T
+typedef int sig_atomic_t;
+#endif
+
 #ifndef SHUT_RD
 #  define SHUT_RD 0
 #endif
@@ -72,7 +80,9 @@
 
 /* SOL_TCP: AIX 4.3.3 */
 #ifndef SOL_TCP
-#  define SOL_TCP IPPROTO_TCP
+#  ifdef IPPROTO_TCP
+#     define SOL_TCP IPPROTO_TCP
+#  endif
 #endif
 
 /* POSIX.1 doesn't seem to know sockets */
@@ -99,7 +109,7 @@
 #  define SIZET_MAX UINT_MAX
 #  define SSIZET_MIN INT_MIN
 #  define SSIZET_MAX INT_MAX
-#  define F_Zd "%d"
+#  define F_Zd "%""d"
 #  define F_Zu "%u"
 #elif HAVE_BASIC_SIZE_T==6
 #  define SIZET_MAX ULONG_MAX
@@ -153,7 +163,7 @@
 #  elif HAVE_BASIC_PID_T==2
 #define F_pid "%hu"
 #  elif HAVE_BASIC_PID_T==3
-#define F_pid "%d"
+#define F_pid "%""d"
 #  elif HAVE_BASIC_PID_T==4
 #define F_pid "%u"
 #  elif HAVE_BASIC_PID_T==5
@@ -177,7 +187,7 @@
 #  elif HAVE_BASIC_UID_T==2
 #define F_uid "%hu"
 #  elif HAVE_BASIC_UID_T==3
-#define F_uid "%d"
+#define F_uid "%""d"
 #  elif HAVE_BASIC_UID_T==4
 #define F_uid "%u"
 #  elif HAVE_BASIC_UID_T==5
@@ -201,7 +211,7 @@
 #  elif HAVE_BASIC_GID_T==2
 #define F_gid "%hu"
 #  elif HAVE_BASIC_GID_T==3
-#define F_gid "%d"
+#define F_gid "%""d"
 #  elif HAVE_BASIC_GID_T==4
 #define F_gid "%u"
 #  elif HAVE_BASIC_GID_T==5
@@ -225,13 +235,17 @@
 #  elif HAVE_BASIC_TIME_T==2
 #define F_time "%hu"
 #  elif HAVE_BASIC_TIME_T==3
-#define F_time "%d"
+#define F_time "%""d"
 #  elif HAVE_BASIC_TIME_T==4
 #define F_time "%u"
 #  elif HAVE_BASIC_TIME_T==5
 #define F_time "%ld"
 #  elif HAVE_BASIC_TIME_T==6
 #define F_time "%lu"
+#  elif HAVE_BASIC_TIME_T==7
+#define F_time "%Ld"
+#  elif HAVE_BASIC_TIME_T==8
+#define F_time "%Lu"
 #  else
 #error "HAVE_BASIC_TIME_T is out of range:" HAVE_BASIC_TIME_T
 #  endif
@@ -249,13 +263,17 @@
 #  elif HAVE_BASIC_SOCKLEN_T==2
 #define F_socklen "%hu"
 #  elif HAVE_BASIC_SOCKLEN_T==3
-#define F_socklen "%d"
+#define F_socklen "%""d"
 #  elif HAVE_BASIC_SOCKLEN_T==4
 #define F_socklen "%u"
 #  elif HAVE_BASIC_SOCKLEN_T==5
 #define F_socklen "%ld"
 #  elif HAVE_BASIC_SOCKLEN_T==6
 #define F_socklen "%lu"
+#  elif HAVE_BASIC_SOCKLEN_T==7
+#define F_socklen "%Ld"
+#  elif HAVE_BASIC_SOCKLEN_T==8
+#define F_socklen "%Lu"
 #  else
 #error "HAVE_BASIC_SOCKLEN_T is out of range:" HAVE_BASIC_SOCKLEN_T
 #  endif
@@ -267,7 +285,7 @@
 #endif
 #ifndef F_off
 #  if HAVE_BASIC_OFF_T==3
-#     define F_off "%d"
+#     define F_off "%""d"
 #  elif HAVE_BASIC_OFF_T==5
 #     define F_off "%ld"
 #  elif HAVE_BASIC_OFF_T==7
@@ -288,7 +306,7 @@
 #  elif HAVE_BASIC_OFF64_T==2
 #define F_off64 "%hu"
 #  elif HAVE_BASIC_OFF64_T==3
-#define F_off64 "%d"
+#define F_off64 "%""d"
 #  elif HAVE_BASIC_OFF64_T==4
 #define F_off64 "%u"
 #  elif HAVE_BASIC_OFF64_T==5
@@ -306,32 +324,61 @@
 
 
 /* all unsigned; default: unsigned long */
-#if !defined(HAVE_TYPEOF_ST_DEV) || !HAVE_TYPEOF_ST_DEV
-#  undef HAVE_TYPEOF_ST_DEV
-#  define HAVE_TYPEOF_ST_DEV 6
+#if !defined(HAVE_BASIC_DEV_T) || !HAVE_BASIC_DEV_T
+#  undef HAVE_BASIC_DEV_T
+#  define HAVE_BASIC_DEV_T 6
 #endif
-#ifndef F_st_dev
-#  if HAVE_TYPEOF_ST_DEV==1
-#define F_st_dev "%hd"
-#  elif HAVE_TYPEOF_ST_DEV==2
-#define F_st_dev "%hu"
-#  elif HAVE_TYPEOF_ST_DEV==3
-#define F_st_dev "%d"
-#  elif HAVE_TYPEOF_ST_DEV==4
-#define F_st_dev "%u"
-#  elif HAVE_TYPEOF_ST_DEV==5
-#define F_st_dev "%ld"
-#  elif HAVE_TYPEOF_ST_DEV==6
-#define F_st_dev "%lu"
-#  elif HAVE_TYPEOF_ST_DEV==7
-#define F_st_dev "%Ld"
-#  elif HAVE_TYPEOF_ST_DEV==8
-#define F_st_dev "%Lu"
+#ifndef F_dev
+#  if HAVE_BASIC_DEV_T==1
+#define F_dev "%hd"
+#  elif HAVE_BASIC_DEV_T==2
+#define F_dev "%hu"
+#  elif HAVE_BASIC_DEV_T==3
+#define F_dev "%""d"
+#  elif HAVE_BASIC_DEV_T==4
+#define F_dev "%u"
+#  elif HAVE_BASIC_DEV_T==5
+#define F_dev "%ld"
+#  elif HAVE_BASIC_DEV_T==6
+#define F_dev "%lu"
+#  elif HAVE_BASIC_DEV_T==7
+#define F_dev "%Ld"
+#  elif HAVE_BASIC_DEV_T==8
+#define F_dev "%Lu"
 #  else
-#error "HAVE_TYPEOF_ST_DEV is out of range:" HAVE_TYPEOF_ST_DEV
+#error "HAVE_BASIC_DEV_T is out of range:" HAVE_BASIC_DEV_T
 #  endif
 #endif
 
+
+#if _WITH_TERMIOS
+#if !defined(HAVE_BASIC_SPEED_T) || !HAVE_BASIC_SPEED_T
+#  undef HAVE_BASIC_SPEED_T
+#  define HAVE_BASIC_SPEED_T 4
+#endif
+#ifndef F_speed
+#  if HAVE_BASIC_SPEED_T==1
+#define F_speed "%hd"
+#  elif HAVE_BASIC_SPEED_T==2
+#define F_speed "%hu"
+#  elif HAVE_BASIC_SPEED_T==3
+#define F_speed "%""d"
+#  elif HAVE_BASIC_SPEED_T==4
+#define F_speed "%u"
+#  elif HAVE_BASIC_SPEED_T==5
+#define F_speed "%ld"
+#  elif HAVE_BASIC_SPEED_T==6
+#define F_speed "%lu"
+#  elif HAVE_BASIC_SPEED_T==7
+#define F_speed "%Ld"
+#  elif HAVE_BASIC_SPEED_T==8
+#define F_speed "%Lu"
+#  else
+#error "HAVE_BASIC_SPEED_T is out of range:" HAVE_BASIC_SPEED_T
+#  endif
+#endif
+#endif /* _WITH_TERMIOS */
+
 /* all unsigned; default; unsigned long */
 #if !defined(HAVE_TYPEOF_ST_INO) || !HAVE_TYPEOF_ST_INO
 #  undef HAVE_TYPEOF_ST_INO
@@ -343,7 +390,7 @@
 #  elif HAVE_TYPEOF_ST_INO==2
 #define F_st_ino "%hu"
 #  elif HAVE_TYPEOF_ST_INO==3
-#define F_st_ino "%d"
+#define F_st_ino "%""d"
 #  elif HAVE_TYPEOF_ST_INO==4
 #define F_st_ino "%u"
 #  elif HAVE_TYPEOF_ST_INO==5
@@ -370,7 +417,7 @@
 #  elif HAVE_TYPEOF_ST64_INO==2
 #define F_st64_ino "%hu"
 #  elif HAVE_TYPEOF_ST64_INO==3
-#define F_st64_ino "%d"
+#define F_st64_ino "%""d"
 #  elif HAVE_TYPEOF_ST64_INO==4
 #define F_st64_ino "%u"
 #  elif HAVE_TYPEOF_ST64_INO==5
@@ -397,13 +444,17 @@
 #  elif HAVE_TYPEOF_ST_NLINK==2
 #define F_st_nlink "%hu"
 #  elif HAVE_TYPEOF_ST_NLINK==3
-#define F_st_nlink "%d"
+#define F_st_nlink "%""d"
 #  elif HAVE_TYPEOF_ST_NLINK==4
 #define F_st_nlink "%u"
 #  elif HAVE_TYPEOF_ST_NLINK==5
 #define F_st_nlink "%ld"
 #  elif HAVE_TYPEOF_ST_NLINK==6
 #define F_st_nlink "%lu"
+#  elif HAVE_TYPEOF_ST_NLINK==7
+#define F_st_nlink "%Ld"
+#  elif HAVE_TYPEOF_ST_NLINK==8
+#define F_st_nlink "%Lu"
 #  else
 #error "HAVE_TYPEOF_ST_NLINK is out of range:" HAVE_TYPEOF_ST_NLINK
 #  endif
@@ -420,7 +471,7 @@
 #  elif HAVE_TYPEOF_ST_SIZE==2
 #define F_st_size "%hu"
 #  elif HAVE_TYPEOF_ST_SIZE==3
-#define F_st_size "%d"
+#define F_st_size "%""d"
 #  elif HAVE_TYPEOF_ST_SIZE==4
 #define F_st_size "%u"
 #  elif HAVE_TYPEOF_ST_SIZE==5
@@ -447,7 +498,7 @@
 #  elif HAVE_TYPEOF_ST64_SIZE==2
 #define F_st64_size "%hu"
 #  elif HAVE_TYPEOF_ST64_SIZE==3
-#define F_st64_size "%d"
+#define F_st64_size "%""d"
 #  elif HAVE_TYPEOF_ST64_SIZE==4
 #define F_st64_size "%u"
 #  elif HAVE_TYPEOF_ST64_SIZE==5
@@ -474,13 +525,17 @@
 #  elif HAVE_TYPEOF_ST_BLKSIZE==2
 #define F_st_blksize "%hu"
 #  elif HAVE_TYPEOF_ST_BLKSIZE==3
-#define F_st_blksize "%d"
+#define F_st_blksize "%""d"
 #  elif HAVE_TYPEOF_ST_BLKSIZE==4
 #define F_st_blksize "%u"
 #  elif HAVE_TYPEOF_ST_BLKSIZE==5
 #define F_st_blksize "%ld"
 #  elif HAVE_TYPEOF_ST_BLKSIZE==6
 #define F_st_blksize "%lu"
+#  elif HAVE_TYPEOF_ST_BLKSIZE==7
+#define F_st_blksize "%Ld"
+#  elif HAVE_TYPEOF_ST_BLKSIZE==8
+#define F_st_blksize "%Lu"
 #  else
 #error "HAVE_TYPEOF_ST_BLKSIZE is out of range:" HAVE_TYPEOF_ST_BLKSIZE
 #  endif
@@ -497,7 +552,7 @@
 #  elif HAVE_TYPEOF_ST_BLOCKS==2
 #define F_st_blocks "%hu"
 #  elif HAVE_TYPEOF_ST_BLOCKS==3
-#define F_st_blocks "%d"
+#define F_st_blocks "%""d"
 #  elif HAVE_TYPEOF_ST_BLOCKS==4
 #define F_st_blocks "%u"
 #  elif HAVE_TYPEOF_ST_BLOCKS==5
@@ -524,7 +579,7 @@
 #  elif HAVE_TYPEOF_ST64_BLOCKS==2
 #define F_st64_blocks "%hu"
 #  elif HAVE_TYPEOF_ST64_BLOCKS==3
-#define F_st64_blocks "%d"
+#define F_st64_blocks "%""d"
 #  elif HAVE_TYPEOF_ST64_BLOCKS==4
 #define F_st64_blocks "%u"
 #  elif HAVE_TYPEOF_ST64_BLOCKS==5
@@ -598,6 +653,33 @@
 #  endif
 #endif
 
+/* default: socklen_t */
+#if !defined(HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN) || !HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN
+#  undef HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN
+#  define HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN HAVE_BASIC_SOCKLEN_T
+#endif
+#ifndef F_cmsg_len
+#  if HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN==1
+#define F_cmsg_len "%""hd"
+#  elif HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN==2
+#define F_cmsg_len "%""hu"
+#  elif HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN==3
+#define F_cmsg_len "%""d"
+#  elif HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN==4
+#define F_cmsg_len "%""u"
+#  elif HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN==5
+#define F_cmsg_len "%""ld"
+#  elif HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN==6
+#define F_cmsg_len "%""lu"
+#  elif HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN==7
+#define F_cmsg_len "%""Ld"
+#  elif HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN==8
+#define F_cmsg_len "%""Lu"
+#  else
+#error "HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN is out of range:" HAVE_TYPEOF_STRUCT_CMSGHDR_CMSG_LEN
+#  endif
+#endif
+
 /* Cygwin 1.3.22 has the prototypes, but not the type... */
 #ifndef HAVE_TYPE_STAT64
 #  undef HAVE_STAT64
@@ -622,34 +704,4 @@
 extern const char *hstrerror(int);
 #endif
 
-/*****************************************************************************/
-/* here are the declarations of compat.c */
-
-#if !HAVE_SIGACTION
-struct sigaction {
-   void (*sa_handler)(int);
-   void (*sa_sigaction)(int, siginfo_t *, void *);
-   sigset_t sa_mask;
-   int sa_flags;
-} ;
-struct siginfo {
-   int si_signo;
-   int si_errno;
-   int si_code;
-   pid_t si_pid;
-   uid_t si_uid;
-   int   si_status;
-   /*clock_t si_utime;*/
-   /*clock_t si_stime;*/
-   sigval_t si_value;
-   int      si_int;
-   void    *si_ptr;
-   void    *si_addr;
-   /*int      si_band;*/
-   /*int      si_fd;*/
-} ;
-extern int sigaction(int signum, const struct sigaction *act,
-		     struct sigaction *oldact);
-#endif /* !HAVE_SIGACTION */
-
 #endif /* !defined(__compat_h_included) */
