@@ -1,5 +1,5 @@
 /* source: xioopen.c */
-/* Copyright Gerhard Rieger 2001-2007 */
+/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* this is the source file of the extended open function */
@@ -37,6 +37,10 @@ const struct addrname addressnames[] = {
    { "creat",	&addr_creat },
    { "create",	&addr_creat },
 #endif
+#if WITH_GENERICSOCKET
+   { "datagram",		&xioaddr_socket_datagram },
+   { "dgram",			&xioaddr_socket_datagram },
+#endif
 #if WITH_PIPE
    { "echo",		&addr_pipe },
 #endif
@@ -55,6 +59,9 @@ const struct addrname addressnames[] = {
 #if WITH_GOPEN
    { "gopen",	&addr_gopen },
 #endif
+#if WITH_INTERFACE
+   { "if",		&xioaddr_interface },
+#endif
 #if (WITH_IP4 || WITH_IP6) && WITH_TCP
    { "inet",		&addr_tcp_connect },
 #endif
@@ -76,6 +83,9 @@ const struct addrname addressnames[] = {
    { "inet6-l",	&addr_tcp6_listen },
    { "inet6-listen",	&addr_tcp6_listen },
 #endif
+#if WITH_INTERFACE
+   { "interface",	&xioaddr_interface },
+#endif
 #if WITH_RAWIP
 #if (WITH_IP4 || WITH_IP6)
    { "ip",		&addr_rawip_sendto },
@@ -106,7 +116,7 @@ const struct addrname addressnames[] = {
 #endif
 #endif /* WITH_RAWIP */
 #if WITH_UNIX
-   { "local",	&addr_unix_connect },
+   { "local",	&xioaddr_unix_connect },
 #endif
 #if WITH_FILE
    { "open",		&addr_open },
@@ -131,6 +141,43 @@ const struct addrname addressnames[] = {
 #if WITH_READLINE
    { "readline",	&addr_readline },
 #endif
+#if (WITH_IP4 || WITH_IP6) && WITH_SCTP
+   { "sctp",		&addr_sctp_connect },
+   { "sctp-connect",	&addr_sctp_connect },
+#if WITH_LISTEN
+   { "sctp-l",		&addr_sctp_listen },
+   { "sctp-listen",	&addr_sctp_listen },
+#endif
+#if WITH_IP4
+   { "sctp4",		&addr_sctp4_connect },
+   { "sctp4-connect",	&addr_sctp4_connect },
+#if WITH_LISTEN
+   { "sctp4-l",		&addr_sctp4_listen },
+   { "sctp4-listen",	&addr_sctp4_listen },
+#endif
+#endif /* WITH_IP4 */
+#if WITH_IP6
+   { "sctp6",		&addr_sctp6_connect },
+   { "sctp6-connect",	&addr_sctp6_connect },
+#if WITH_LISTEN
+   { "sctp6-l",		&addr_sctp6_listen },
+   { "sctp6-listen",	&addr_sctp6_listen },
+#endif
+#endif /* WITH_IP6 */
+#endif /* (WITH_IP4 || WITH_IP6) && WITH_SCTP */
+#if WITH_GENERICSOCKET
+   { "sendto",			&xioaddr_socket_sendto },
+#endif
+#if WITH_GENERICSOCKET
+   { "socket-connect",		&xioaddr_socket_connect },
+   { "socket-datagram",		&xioaddr_socket_datagram },
+#if WITH_LISTEN
+   { "socket-listen",		&xioaddr_socket_listen },
+#endif /* WITH_LISTEN */
+   { "socket-recv",		&xioaddr_socket_recv },
+   { "socket-recvfrom",		&xioaddr_socket_recvfrom },
+   { "socket-sendto",		&xioaddr_socket_sendto },
+#endif
 #if WITH_SOCKS4
    { "socks",	&addr_socks4_connect },
    { "socks4",	&addr_socks4_connect },
@@ -231,19 +278,19 @@ const struct addrname addressnames[] = {
    { "udp6-sendto",	&addr_udp6_sendto },
 #endif
 #if WITH_UNIX
-   { "unix",		&addr_unix_client },
-   { "unix-client",	&addr_unix_client },
-   { "unix-connect",	&addr_unix_connect },
+   { "unix",		&xioaddr_unix_client },
+   { "unix-client",	&xioaddr_unix_client },
+   { "unix-connect",	&xioaddr_unix_connect },
 #endif
 #if WITH_UNIX && WITH_LISTEN
-   { "unix-l",		&addr_unix_listen },
-   { "unix-listen",	&addr_unix_listen },
+   { "unix-l",		&xioaddr_unix_listen },
+   { "unix-listen",	&xioaddr_unix_listen },
 #endif
 #if WITH_UNIX
-   { "unix-recv",	&addr_unix_recv },
-   { "unix-recvfrom",	&addr_unix_recvfrom },
-   { "unix-send",	&addr_unix_sendto },
-   { "unix-sendto",	&addr_unix_sendto },
+   { "unix-recv",	&xioaddr_unix_recv },
+   { "unix-recvfrom",	&xioaddr_unix_recvfrom },
+   { "unix-send",	&xioaddr_unix_sendto },
+   { "unix-sendto",	&xioaddr_unix_sendto },
 #endif
 #else /* !0 */
 #  if WITH_INTEGRATE
@@ -296,20 +343,15 @@ static xiofile_t *xioallocfd(void) {
 
    fd->stream.fd        = -1;
    fd->stream.dtype     = XIODATA_STREAM;
-#if WITH_SOCKET
+#if _WITH_SOCKET
 /* fd->stream.salen     = 0; */
-#endif /* WITH_SOCKET */
+#endif /* _WITH_SOCKET */
    fd->stream.howtoend  = END_UNSPEC;
 /* fd->stream.name      = NULL; */
+   fd->stream.escape	= -1;
 /* fd->stream.para.exec.pid = 0; */
    fd->stream.lineterm  = LINETERM_RAW;
 
-   /*!! support n socks */
-   if (!sock[0]) {
-      sock[0] = fd;
-   } else {
-      sock[1] = fd;
-   }
    return fd;
 }
 
@@ -328,6 +370,12 @@ xiofile_t *xioopen(const char *addr,	/* address specification */
    if ((xfd = xioparse_dual(&addr)) == NULL) {
       return NULL;
    }
+   /*!! support n socks */
+   if (!sock[0]) {
+      sock[0] = xfd;
+   } else {
+      sock[1] = xfd;
+   }
    if (xioopen_dual(xfd, xioflags) < 0) {
       /*!!! free something? */
       return NULL;
